@@ -1,7 +1,7 @@
 /*
  * Alternate global C function prototypes for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 /* command function list */
@@ -9,7 +9,7 @@ int		clear_cmd(), quit_cmd(), list_cmd(), simplify_cmd(), help_cmd(), eliminate_
 int		fraction_cmd(), unfactor_cmd(), compare_cmd(), extrema_cmd();
 int		read_cmd(), display_cmd(), calculate_cmd(), solve_cmd();
 int		factor_cmd(), derivative_cmd(), replace_cmd(), approximate_cmd();
-int		save_cmd(), taylor_cmd(), limit_cmd(), echo_cmd();
+int		save_cmd(), taylor_cmd(), limit_cmd(), echo_cmd(), plot_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd(), version_cmd();
 int		edit_cmd(), real_cmd(), imaginary_cmd(), tally_cmd();
 int		roots_cmd(), set_cmd(), variables_cmd(), code_cmd(), optimize_cmd(), push_cmd();
@@ -27,6 +27,4 @@ long		decstrtol();
 void fphandler(int sig);
 void inthandler(int sig);
 void alarmhandler(int sig);
-void get_screen_size(void);
 void resizehandler(int sig);
-void exit_program(int exit_value);
