@@ -1,10 +1,14 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
+#if	UNIX || CYGWIN
+#include <sys/ioctl.h>
+#include <termios.h>
+#endif
 
 /*
  * Standard function to report an error to the user.
@@ -59,6 +63,12 @@ error_bug(str)
 const char	*str;	/* constant string to display */
 {
 	error(str);		/* Display the passed error message. */
+#if	SILENT
+	printf("%s\n", str);
+#endif
+	printf(_("Please report this bug to the maintainers,\n"));
+	printf(_("along with the entry sequence that caused it.\n"));
+	printf(_("Type \"help bugs\" for info on how to report bugs found in this program.\n"));
 	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
 }
 
@@ -88,12 +98,40 @@ check_err()
 	}
 }
 
+#if	UNIX || CYGWIN
+/*
+ * Get the screen (window) width and height from the operating system.
+ *
+ * Sets the global integers screen_columns and screen_rows.
+ */
+void
+get_screen_size()
+{
+	struct winsize	ws;
+
+	ws.ws_col = 0;
+	ws.ws_row = 0;
+	if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
+		if (ws.ws_col > 0) {
+			screen_columns = ws.ws_col;
+		}
+		if (ws.ws_row > 0) {
+			screen_rows = ws.ws_row;
+		}
+	}
+}
+#endif
+
 /*
  * Allocate the needed global expression storage arrays.
- * Each has "n_tokens" elements.
- * "n_tokens" must not change until Mathomatic terminates.
+ * Each has n_tokens elements.
+ * n_tokens must not change until Mathomatic terminates.
  *
- * Return true if successful.
+ * init_mem() is only called once upon Mathomatic startup
+ * before using the symbolic math engine,
+ * and can be undone by calling free_mem() below.
+ *
+ * Returns true if successful, otherwise Mathomatic cannot be used.
  */
 int
 init_mem()
@@ -107,9 +145,46 @@ init_mem()
 	if (alloc_next_espace() < 0) {	/* make sure there is at least 1 equation space */
 		return false;
 	}
+	clear_all();
 	return true;
 }
 
+#if	false
+/*
+ * Free the global expression storage arrays.
+ * After calling this, memory usage is reset and Mathomatic becomes unusable,
+ * so do not call unless finished with using the Mathomatic code.
+ *
+ * This routine is not needed, because when a program exits,
+ * all the memory it allocated is released by the operating system.
+ * Inclusion of this routine was requested by Tam Hanna.
+ */
+void
+free_mem()
+{
+	int i;
+
+	clear_all();
+
+	free(scratch);
+	free(tes);
+	free(tlhs);
+	free(trhs);
+
+	for (i = 0; i < N_EQUATIONS; i++) {
+		if (lhs[i]) {
+			free(lhs[i]);
+			lhs[i] = NULL;
+		}
+		if (rhs[i]) {
+			free(rhs[i]);
+			rhs[i] = NULL;
+		}
+	}
+	n_equations = 0;
+}
+#endif
+
 /*
  * Initialize some important global variables to their defaults.
  * This is called on startup and by process() to reset the global flags to the default state.
@@ -123,6 +198,7 @@ init_gvars()
 	symb_flag = false;
 	sign_cmp_flag = false;
 	approximate_roots = false;
+	repeat_flag = false;
 
 	/* initialize the universal and often used constant "0" expression */
 	zero_token.level = 1;
@@ -144,11 +220,12 @@ clean_up()
 	int	i;
 
 	init_gvars();		/* reset the global variables to the default */
-	if (gfp != stdout) {	/* reset the output file to standard output */
+	if (gfp != default_out) {	/* reset the output file to default */
 #if	!SECURE
-		fclose(gfp);
+		if (gfp != stdout && gfp != stderr)
+			fclose(gfp);
 #endif
-		gfp = stdout;
+		gfp = default_out;
 	}
 	for (i = 0; i < n_equations; i++) {
 		if (n_lhs[i] <= 0) {
@@ -377,6 +454,8 @@ long	v;	/* standard Mathomatic variable */
 		return false;
 	if (found_var(lhs[i], n_lhs[i], v))
 		return true;
+	if (n_rhs[i] <= 0)
+		return false;
 	if (found_var(rhs[i], n_rhs[i], v))
 		return true;
 	return false;
@@ -443,6 +522,7 @@ int		n;		/* expression length */
 /*
  * Get default equation number from a command parameter string.
  * The equation number must be the only parameter.
+ * If no equation number is specified, default to the current equation.
  *
  * Return -1 on error.
  */
@@ -627,7 +707,7 @@ int	n;		/* maximum size of "string" in bytes */
 	if (i >= 0 && string[i] == '\n') {
 		string[i] = '\0';
 	}
-	if (gfp != stdout || echo_input) {
+	if ((gfp != stdout && gfp != stderr) || echo_input) {
 		/* Input that is prompted for is now included in the redirected output
 		   of a command to a file, making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
@@ -678,19 +758,32 @@ int	en;	/* equation space number the result is in */
 		return false;
 	}
 #if	LIBRARY
+	make_fractions_and_group(en);
+	if (factor_int_flag) {
+		factor_int_sub(en);
+	}
+	free_result_str();
+	result_str = list_equation(en, false);
 	if (gfp == stdout) {
-		make_fractions_and_group(en);
-		if (factor_int_flag) {
-			factor_int_sub(en);
-		}
-		free(result_str);
-		result_str = list_equation(en, false);
 		return(result_str != NULL);
 	}
 #endif
 	return(list_sub(en) != 0);
 }
 
+/*
+ * Free any malloc()ed result_str, so there won't be a memory leak
+ * in the symbolic math library.
+ */
+void
+free_result_str()
+{
+	if (result_str) {
+		free(result_str);
+		result_str = NULL;
+	}
+}
+
 /*
  * Return true if the first word in the passed string is "all".
  */
@@ -784,7 +877,7 @@ char	*cp;	/* command line string */
 	if (cp) {
 		cp = skip_space(cp);
 		if (*cp) {
-			error(_("Extra characters encountered."));
+			error(_("Extra characters or unrecognized argument."));
 			return true;
 		}
 	}
@@ -991,7 +1084,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == CONSTANT && !isfinite(p1[i].token.constant)) {
 			return true;
 		}
