@@ -1,7 +1,7 @@
 /*
  * Main include file for Mathomatic.  Can be edited.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #ifndef	max
@@ -15,19 +15,23 @@
 #define INFINITY	HUGE_VAL			/* the floating point, positive infinity constant */
 #endif
 #ifndef	isfinite
-#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity or NaN) */
+#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity nor NaN) */
+#endif
+
+#ifndef PATH_MAX					/* make sure the maximum filename length is set */
+#define PATH_MAX	_POSIX_PATH_MAX
 #endif
 
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 #define CLEAR_ARRAY(a)	memset(a, 0, sizeof(a))			/* sets all elements of an array to zero */
 
-#define	MAX_K_INTEGER	1.0e15				/* maximum safely representable integer, 15 digits for doubles */
+#define	MAX_K_INTEGER	1.0e15		/* maximum safely representable floating point integer, 15 digits for doubles */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
 #if	I18N		/* internationalization: output in languages other than English using gettext(3) */
-#define _(str)		gettext(str)	/* currently the support code for this incomplete and untested */
+#define _(str)		gettext(str)
 #else
 #define _(str)		str		/* constant strings to be translated should be marked with this macro */
 #endif
@@ -40,7 +44,7 @@
 #define	PROMPT_STR	"-> "				/* user interface main prompt strings, preceded by the current equation number */
 #define	HTML_PROMPT_STR	"&mdash;&gt; "
 
-#define	MAX_CMD_LEN	(max(PATH_MAX, 500))		/* maximum command line length (not expression input), also max filename length */
+#define	MAX_CMD_LEN	(max(PATH_MAX, 512))		/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
@@ -131,6 +135,14 @@ typedef struct {		/* qsort(3) data structure for sorting Mathomatic variables */
 	int	count;		/* number of times the variable occurs */
 } sort_type;
 
+/* A list of supported output languages as described */
+enum language_list {
+  C = 1,      /* or C++ */
+  JAVA = 2,
+  PYTHON = 3
+};
+
+
 /* Debugging macros: */
 #if	SILENT
 #define	list_tdebug(level)
