@@ -1,12 +1,12 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
+#define	OPT_MIN_SIZE	7	/* Minimum size (in tokens) of repeated expressions to find in optimize command. */
 
 static int	sum_product();
 static int	complex_func();
@@ -16,6 +16,64 @@ static int	elim_sub();
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
 
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+/*
+ * The plot command.
+ */
+int
+plot_cmd(cp)
+char	*cp;	/* the command line argument */
+{
+	char	cl[MAX_CMD_LEN];
+	int	free_cp = false;
+
+	if (security_level) {
+		error(_("Command disabled."));
+		return false;
+	}
+	if (*cp == '\0') {
+		if (current_not_defined()) {
+			return false;
+		}
+		if (n_rhs[cur_equation]) {
+			cp = list_expression(rhs[cur_equation], n_rhs[cur_equation], 3);
+		} else {
+			cp = list_expression(lhs[cur_equation], n_lhs[cur_equation], 3);
+		}
+		if (cp) {
+			free_cp = true;
+		} else {
+			return false;
+		}
+	}
+	if (strpbrk(cp, "'\\")) {
+		error(_("Invalid character in expression string."));
+		if (free_cp)
+			free(cp);
+		return false;
+	}
+	if (strchr(cp, 'y')) {
+		printf(_("Performing 3D surface plot...\n"));
+		snprintf(cl, sizeof(cl), "echo 'splot %s'|gnuplot -persist", cp);
+	} else {
+		printf(_("Performing 2D plot...\n"));
+		snprintf(cl, sizeof(cl), "echo 'plot %s'|gnuplot -persist", cp);
+	}
+	if (free_cp)
+		free(cp);
+	if (shell_out(cl)) {
+		error(_("Error trying to run gnuplot."));
+		printf(_("Command line = \"%s\".\n"), cl);
+		return false;
+#if	!SILENT
+	} else if (debug_level > 0) {
+		printf(_("Command line = \"%s\".\n"), cl);
+#endif
+	}
+	return true;
+}
+#endif
+
 /*
  * The version command.
  */
@@ -25,12 +83,18 @@ char	*cp;	/* the command line argument */
 {
 	int		rv;	/* return value */
 
-	extra_characters(cp);	/* no command options allowed */
 #if	LIBRARY
-	free(result_str);
+	free_result_str();
 	result_str = strdup(VERSION);
 #endif
 	rv = version_report();
+
+	fprintf(gfp, _("\nMathomatic is GNU LGPL version 2.1 licensed software,\n"));
+	fprintf(gfp, _("meaning it is free software that comes with no warranty.\n"));
+	fprintf(gfp, _("Type \"help copyright\" for the copyright and license.\n"));
+
+	fprintf(gfp, _("\nThe newest version of Mathomatic is always available at\n"));
+	fprintf(gfp, _("the Mathomatic website: http://mathomatic.org\n"));
 	return rv;
 }
 
@@ -41,7 +105,7 @@ int
 version_report()
 {
 	fprintf(gfp, _("Mathomatic version %s\n"), VERSION);
-	fprintf(gfp, _("Compile defines used: "));
+	fprintf(gfp, _("Compile-time defines used: "));
 #if	UNIX
 	fprintf(gfp, "UNIX ");
 #endif
@@ -69,21 +133,24 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, _("\nThe current expression array size is %d tokens,\n"), n_tokens);
+#if	NO_COLOR
+	fprintf(gfp, "NO_COLOR ");
+#endif
+	fprintf(gfp, _("\n\nThe current expression array size is %d tokens,\n"), n_tokens);
 	fprintf(gfp, _("making the maximum memory usage %ld kilobytes.\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
-
-#if	!SECURE
-	fprintf(gfp, _("The current security_level is %d.\n"), security_level);
+#if	SECURE
+	fprintf(gfp, _("Compiled for maximum security.\n"));
+#else
+	fprintf(gfp, _("The current security level is %d.\n"), security_level);
 #endif
-
-	fprintf(gfp, _("\nMathomatic is GNU LGPL version 2.1 licensed software,\n"));
-	fprintf(gfp, _("meaning it is free software that comes with no warranty.\n"));
-	fprintf(gfp, "Type \"help copyright\" for copyright and license information.\n");
 	return true;
 }
 
 /*
  * The solve command.
+ *
+ * Return 0 on failure, 1 on total success,
+ * or 2 if partial success (solutions didn't totally verify when doing "solve verify").
  */
 int
 solve_cmd(cp)
@@ -94,7 +161,7 @@ char	*cp;
 	long		v;	/* Mathomatic variable */
 	token_type	want;
 	int		diff_sign;
-	int		verify_flag = false, plural_flag = false, once_through;
+	int		verify_flag = false, plural_flag = false, once_through, contains_infinity;
 	char		*cp1;
 
 	if (cur_equation < 0 || cur_equation >= n_equations || n_lhs[cur_equation] <= 0 || n_rhs[cur_equation] <= 0) {
@@ -130,7 +197,7 @@ char	*cp;
 		if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) > 0) {
 			return_result(cur_equation);
 			if (compare_es(cur_equation, i)) {
-				warning(_("Nothing was done, so no need to verify."));
+				debug_string(0, _("Nothing was done, so no need to verify solution."));
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 				return true;
@@ -147,18 +214,18 @@ char	*cp;
 			calc_simp(rhs[i], &n_rhs[i]);
 			once_through = false;
 check_result:
-			if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
-#if	!SILENT
+			contains_infinity = (exp_contains_infinity(lhs[i], n_lhs[i])
+			    || exp_contains_infinity(rhs[i], n_rhs[i]));
+                        if (!contains_infinity && se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
 				if (plural_flag)
 					printf(_("Solutions verified.\n"));
 				else
 					printf(_("Solution verified.\n"));
-#endif
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 				return true;
 			} else {
-				if (!once_through) {
+				if (!contains_infinity && !once_through) {
 					symb_flag = symblify;
 					simpa_side(lhs[i], &n_lhs[i], true, false);
 					simpa_side(rhs[i], &n_rhs[i], true, false);
@@ -166,12 +233,12 @@ check_result:
 					once_through = true;
 					goto check_result;
 				}
-				error(_("Solution may be incorrect."));
+				error(_("Solution might be incorrect."));
 #if	true	/* set to false for testing */
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 #endif
-				return false;
+				return 2;
 			}
 		} else {
 #if	!SILENT
@@ -462,7 +529,7 @@ int		*np;
 					k1 = i1 - 1;
 					if ((jj1 - k1) >= OPT_MIN_SIZE
 					    && se_compare(&equation[k], j - k, &equation[k1], jj1 - k1, &diff_sign)) {
-						snprintf(var_name_buf, sizeof(var_name_buf), "temp%d", last_temp_var);
+						snprintf(var_name_buf, sizeof(var_name_buf), "temp%.0d", last_temp_var);
 						if (parse_var(&v, var_name_buf) == NULL) {
 							return false;	/* can't create "temp" variable */
 						}
@@ -613,7 +680,7 @@ char	*cp;
 	int	i, j;
 
 	if (!readline_enabled) {
-		error(_("Readline is disabled."));
+		error(_("Readline is currently turned off."));
 		return false;
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -638,10 +705,12 @@ char	*cp;
  */
 int
 push_en(en)
-int	en;	/* equation space number */
+int	en;	/* equation space number to push */
 {
 	char	*cp;
 
+	if (!readline_enabled)
+		return false;
 	high_prec = true;
 	cp = list_equation(en, false);
 	high_prec = false;
@@ -663,7 +732,7 @@ output_current_directory(ofp)
 FILE	*ofp;	/* output file pointer */
 {
 #if	!SECURE
-	char	buf[MAX_CMD_LEN];
+	char	buf[PATH_MAX];
 
 	if (security_level < 3 && ofp && getcwd(buf, sizeof(buf))) {
 		fprintf(ofp, "directory = %s\n", buf);
@@ -724,11 +793,6 @@ FILE	*ofp;	/* output file pointer */
 	}
 	fprintf(ofp, "display2d\n");
 
-	if (quiet_mode) {
-		fprintf(ofp, "no ");
-	}
-	fprintf(ofp, "prompt\n");
-
 	if (!preserve_surds) {
 		fprintf(ofp, "no ");
 	}
@@ -828,11 +892,12 @@ char	*cp;
 	int	negate;
 	char	*cp1, *option_string;
 
+try_next_param:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-#if	!SECURE
+#if	!SECURE && !LIBRARY
 	if (security_level < 3 && strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
 		if (*cp == '\0') {
@@ -890,6 +955,20 @@ char	*cp;
 		}
 		goto check_return;
 	}
+	if (strncasecmp(option_string, "wide", 4) == 0) {
+		if (!negate) {
+			screen_columns = 0;
+			screen_rows = 0;
+		} else {
+#if	UNIX || CYGWIN
+			get_screen_size();
+#else
+			screen_columns = STANDARD_SCREEN_COLUMNS;
+			screen_rows = STANDARD_SCREEN_ROWS;
+#endif
+		}
+		goto try_next_param;
+	}
 	if (strncasecmp(option_string, "precision", 4) == 0) {
 		i = decstrtol(cp, &cp1);
 		if (i < 0 || i > 14 || cp == cp1) {
@@ -901,39 +980,43 @@ char	*cp;
 	}
 	if (strcmp_tospace(option_string, "auto") == 0) {
 		autosolve = autocalc = autoselect = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "autosolve", 9) == 0) {
 		autosolve = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "autocalc", 8) == 0) {
 		autocalc = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "autoselect", 10) == 0) {
 		autoselect = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "case", 4) == 0) {
 		case_sensitive_flag = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "display2d", 7) == 0) {
 		display2d = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "prompt", 6) == 0) {
 		quiet_mode = negate;
-		goto check_return;
+		goto try_next_param;
+	}
+	if (strncasecmp(option_string, "html", 4) == 0) {
+		html_flag = !negate;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "preserve", 8) == 0) {
 		preserve_surds = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "rationalize", 11) == 0) {
 		rationalize_denominators = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "modulus_mode", 3) == 0) {
 		if (negate) {
@@ -954,14 +1037,14 @@ char	*cp;
 		goto check_return;
 	}
 	if (strncasecmp(option_string, "color", 5) == 0) {
+		reset_attr();
 		color_flag = !negate;
-		cur_color = -1;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "bold", 4) == 0) {
+		reset_attr();
 		bold_colors = !negate;
-		cur_color = -1;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "finance", 7) == 0) {
 		if (negate) {
@@ -987,15 +1070,15 @@ char	*cp;
 	}
 	if (strncasecmp(option_string, "factor_integers", 6) == 0) {
 		factor_int_flag = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "right_associative_power", 5) == 0) {
 		right_associative_power = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "negate_highest_precedence", 6) == 0) {
 		negate_highest_precedence = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
 #if	!SECURE && !LIBRARY
 	if (security_level < 2 && strcmp_tospace(option_string, "save") == 0) {
@@ -1322,7 +1405,7 @@ char	*cp;
 	int		n;
 	int		diff_sign;
 	char		buf[MAX_CMD_LEN];
-	int		factor_flag = false, repeat_flag = false, value_entered;
+	int		factor_flag = false, value_entered;
 
 	if (current_not_defined()) {
 		return false;
@@ -1660,7 +1743,7 @@ char	*cp;
 			fprintf(gfp, _("Expressions differ.\n"));
 			return false;
 		}
-		error(_("Cannot compare an expression with an equation."));
+		error(_("Cannot compare an equation with a non-equation."));
 		return false;
 	}
 	if (compare_es(i, j)) {
@@ -1738,7 +1821,33 @@ times_neg1:
 	}
 	list_proc(lhs[j], n_lhs[j], false);
 	fprintf(gfp, _(") in the second equation.\n"));
-	return(!diff_sign);
+	if (diff_sign)
+		return 0;	/* equations not identical */
+	else
+		return 2;	/* equations identical, but LHS variable name differs */
+}
+
+/*
+ * Display the specified floating point value.
+ * If it is equal to a simple fraction, display that too.
+ *
+ * Return true if a fraction was displayed.
+ */
+int
+display_fraction(value)
+double	value;
+{
+	double	d4, d5;
+	int	rv = false;
+
+	f_to_fraction(value, &d4, &d5);
+	fprintf(gfp, "%.*g", precision, value);
+	if (d5 != 1.0) {
+		fprintf(gfp, " = %.*g/%.*g", precision, d4, precision, d5);
+		rv = true;
+	}
+	fprintf(gfp, "\n");
+	return rv;
 }
 
 /*
@@ -1750,7 +1859,7 @@ char	*cp;
 {
 	long		v = 0, v_tmp;		/* Mathomatic variables */
 	int		i, j;
-	int		nl, nr;
+	int		nleft, nright;
 	double		lcm, d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
@@ -1764,25 +1873,26 @@ char	*cp;
 			return false;
 	}
 	i = next_espace();
+do_repeat:
 /* prompt for the two operands */
 	my_strlcpy(prompt_str, _("Enter dividend: "), sizeof(prompt_str));
-	if (!get_expr(rhs[i], &nr)) {
-		return false;
+	if (!get_expr(rhs[i], &nright)) {
+		return repeat_flag;
 	}
 	my_strlcpy(prompt_str, _("Enter divisor: "), sizeof(prompt_str));
-	if (!get_expr(lhs[i], &nl)) {
-		return false;
+	if (!get_expr(lhs[i], &nleft)) {
+		return repeat_flag;
 	}
 	fprintf(gfp, "\n");
 /* simplify the operand expressions */
-	calc_simp(rhs[i], &nr);
-	calc_simp(lhs[i], &nl);
+	calc_simp(rhs[i], &nright);
+	calc_simp(lhs[i], &nleft);
 /* if division by zero, display a warning */
-	if (get_constant(lhs[i], nl, &d2)) {
+	if (get_constant(lhs[i], nleft, &d2)) {
 		check_divide_by_zero(d2);
 	}
 /* Do constant division if 2 normal numbers were entered */
-	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
+	if (get_constant(rhs[i], nright, &d1) && get_constant(lhs[i], nleft, &d2)) {
 		d3 = gcd_verified(d1, d2);
 		d4 = modf(d1 / d2, &d5);
 		fprintf(gfp, "%.*g/%.*g = %.*g", precision, d1, precision, d2, precision, d1 / d2);
@@ -1794,6 +1904,8 @@ char	*cp;
 		d2 = fabs(d2);
 		if (d3 == 0.0) {
 			fprintf(gfp, _("No GCD found.\n"));
+			if (repeat_flag)
+				goto do_repeat;
 			return true;
 		}
 		fprintf(gfp, "GCD = ");
@@ -1809,46 +1921,52 @@ char	*cp;
 		} else {
 			display_fraction(lcm);
 		}
+		if (repeat_flag)
+			goto do_repeat;
 		return true;
 	}
 /* else do complex number division if 2 complex numbers were entered */
-	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
+	if (parse_complex(rhs[i], nright, &c1) && parse_complex(lhs[i], nleft, &c2)) {
 		c3 = complex_div(c1, c2);
 		fprintf(gfp, _("Result of complex number division:\n"));
 		fprintf(gfp, "%.*g %+.*g*i\n\n", precision, c3.re, precision, c3.im);
+		if (repeat_flag)
+			goto do_repeat;
 		return true;
 	}
 /* else do polynomial division and univariate GCD display */
 	v_tmp = v;
-	if (poly_div(rhs[i], nr, lhs[i], nl, &v_tmp)) {
+	if (poly_div(rhs[i], nright, lhs[i], nleft, &v_tmp)) {
 		simp_divide(tlhs, &n_tlhs);
 		simp_divide(trhs, &n_trhs);
 		list_var(v_tmp, 0);
 		fprintf(gfp, _("Polynomial division successful using base variable (%s).\n"), var_str);
 		fprintf(gfp, _("The quotient is:\n"));
-		display_fractions_and_group(tlhs, &n_tlhs);
+		fractions_and_group(tlhs, &n_tlhs);
 		list_factor(tlhs, &n_tlhs, false);
 		fprintf(gfp, _("\n\nThe remainder is:\n"));
-		display_fractions_and_group(trhs, &n_trhs);
+		fractions_and_group(trhs, &n_trhs);
 		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("Polynomial division failed.\n"));
 	}
 	fprintf(gfp, "\n");
-	j = poly_gcd(rhs[i], nr, lhs[i], nl, v);
+	j = poly_gcd(rhs[i], nright, lhs[i], nleft, v);
 	if (!j) {
-		j = poly_gcd(lhs[i], nl, rhs[i], nr, v);
+		j = poly_gcd(lhs[i], nleft, rhs[i], nright, v);
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
 		fprintf(gfp, _("Polynomial GCD (%d Euclidean algorithm iteration%s):\n"), j, (j == 1) ? "" : "s");
-		display_fractions_and_group(trhs, &n_trhs);
+		fractions_and_group(trhs, &n_trhs);
 		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("No univariate polynomial GCD found.\n"));
 	}
+	if (repeat_flag)
+		goto do_repeat;
 	return true;
 }
 
@@ -1862,7 +1980,7 @@ char	*cp;
 	long	v, last_v, v1, va[MAX_VARS];		/* Mathomatic variables */
 	int	vc = 0;					/* variable count */
 	int	i = 0, n;
-	int	success_flag = false, did_something = false, repeat_flag = false, using_flag;
+	int	success_flag = false, did_something = false, using_flag;
 	char	used[N_EQUATIONS];
 	char	*cp_start;
 	char	buf[MAX_CMD_LEN];
@@ -2037,7 +2155,8 @@ char	*cp;
 {
 	int	i, j;
 	char	*cp1;
-	int	factor_flag;
+	jmp_buf	save_save;
+	int	factor_flag, displayed = false;
 
 	factor_flag = (strcmp_tospace(cp, "factor") == 0);
 	if (factor_flag) {
@@ -2054,22 +2173,34 @@ char	*cp;
 		}
 		for (; i <= j; i++) {
 			if (n_lhs[i] > 0) {
+				blt(save_save, jmp_save, sizeof(jmp_save));
+				if (setjmp(jmp_save) != 0) {	/* trap errors */
+					blt(jmp_save, save_save, sizeof(jmp_save));
+					printf("Skipping equation number %d.\n", i + 1);
+					continue;
+				}
 				make_fractions_and_group(i);
 				if (factor_flag || factor_int_flag) {
 					factor_int_sub(i);
 				}
+				blt(jmp_save, save_save, sizeof(jmp_save));
 #if     LIBRARY
-	                        if (gfp == stdout) {
-	                                free(result_str);
-	                                result_str = list_equation(i, false);
-	                                return(result_str != NULL);
+				free_result_str();
+				result_str = list_equation(i, false);
+				if (result_str == NULL)
+					return false;
+				if (gfp == stdout) {
+	                                displayed = true;
+					continue;
 	                        }
 #endif
-				flist_equation(i);
+				if (flist_equation(i) > 0) {
+					displayed = true;
+				}
 			}
 		}
 	} while (*cp);
-	return true;
+	return displayed;
 }
 
 /*
@@ -2083,7 +2214,10 @@ char	*cp;
 	char	*cp1;
 	int	export_flag = 0;
 
-	if (strncasecmp(cp, "export", 4) == 0) {
+	if (strncasecmp(cp, "gnuplot", 4) == 0) {
+		export_flag = 3;
+		cp = skip_param(cp);
+	} else if (strncasecmp(cp, "export", 4) == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
 	} else if (strncasecmp(cp, "maxima", 4) == 0) {
@@ -2100,14 +2234,18 @@ char	*cp;
 			return false;
 		}
 		for (; i <= j; i++) {
+			if (n_lhs[i] > 0) {
 #if	LIBRARY
-			if (gfp == stdout) {
-				free(result_str);
+				free_result_str();
 				result_str = list_equation(i, export_flag);
-				return(result_str != NULL);
-			}
+				if (result_str == NULL)
+					return false;
+				if (gfp == stdout) {
+					continue;
+				}
 #endif
-			list1_sub(i, export_flag);
+				list1_sub(i, export_flag);
+			}
 		}
 	} while (*cp);
 	return true;
@@ -2120,22 +2258,22 @@ int
 code_cmd(cp)
 char	*cp;
 {
-	int	i, j;
-	int	language = 0;
-	int	int_flag = false, displayed = false;
-	char	*cp1;
+	int			i, j;
+	enum language_list	language = C;
+	int			int_flag = false, displayed = false;
+	char			*cp1;
 
 	for (;; cp = skip_param(cp)) {
 		if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
-			language = 0;
+			language = C;
 			continue;
 		}
 		if (strcmp_tospace(cp, "java") == 0) {
-			language = 1;
+			language = JAVA;
 			continue;
 		}
 		if (strcmp_tospace(cp, "python") == 0) {
-			language = 2;
+			language = PYTHON;
 			continue;
 		}
 		if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
@@ -2200,30 +2338,47 @@ int
 variables_cmd(cp)
 char	*cp;
 {
-	int		i, j, k;
-	int		i1;
-	long		v1, last_v;		/* Mathomatic variables */
-	int		vc, cnt;		/* variable counts */
-	sort_type	va[MAX_VARS];
-	token_type	*p1;
-	int		n1;
-	int		lang_code = 0;
-	int		int_flag = false;
+	int			i, j, k;
+	int			i1;
+	long			v1, last_v;		/* Mathomatic variables */
+	int			vc, cnt;		/* variable counts */
+	sort_type		va[MAX_VARS];
+	token_type		*p1;
+	int			n1;
+	enum language_list	lang_code = 0;		/* default to no programming language */
+	int			int_flag = false, imag_flag = false;
 
 	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
 		cp = skip_param(cp);
-		lang_code = 1;
+		lang_code = C;
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
-		lang_code = 2;
+		lang_code = JAVA;
 	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
 		cp = skip_param(cp);
-		lang_code = 1;
+		lang_code = C;
 		int_flag = true;
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
+	for (k = i; k <= j; k++) {
+		if (n_lhs[k] <= 0)
+			continue;
+		if (n_rhs[k] > 0) {
+			p1 = rhs[k];
+			n1 = n_rhs[k];
+		} else {
+			p1 = lhs[k];
+			n1 = n_lhs[k];
+		}
+		for (i1 = 0; i1 < n1; i1 += 2) {
+			if (p1[i1].kind == VARIABLE && p1[i1].token.variable == IMAGINARY) {
+				imag_flag = true;
+				break;
+			}
+		}
+	}
 	last_v = 0;
 	for (vc = 0;;) {
 		if (vc >= ARR_CNT(va)) {
@@ -2276,10 +2431,13 @@ char	*cp;
 		}
 		list_var(va[i1].v, lang_code);
 		if (lang_code) {
+			if (imag_flag) {
+				fprintf(gfp, "_Complex ");
+			}
 			if (int_flag) {
-				fprintf(gfp, "int\t%s;\n", var_str);
+				fprintf(gfp, "int %s;\n", var_str);
 			} else {
-				fprintf(gfp, "double\t%s;\n", var_str);
+				fprintf(gfp, "double %s;\n", var_str);
 			}
 		} else {
 			fprintf(gfp, "%s\n", var_str);
@@ -2313,6 +2471,10 @@ char	*cp;
 				subst_constants(rhs[i], &n_rhs[i]);
 				approximate_roots = true;
 				simp_sub(i);
+				factorv(lhs[i], &n_lhs[i], IMAGINARY);
+				if (n_rhs[i]) {
+					factorv(rhs[i], &n_rhs[i], IMAGINARY);
+				}
 				approximate_roots = false;
 				if (!return_result(i)) {
 					return false;
@@ -2485,8 +2647,8 @@ char	*cp;
 			if (quickest_flag) {
 				simp_sub(i);
 			} else {
-				simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
-				simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
+				simpa_repeat_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
+				simpa_repeat_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
 			}
 			if (!return_result(i)) {
 				symb_flag = false;
@@ -2568,8 +2730,8 @@ char	*cp;
 				if (quickest_flag) {
 					simp_sub(i1);
 				} else {
-					simpa_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
-					simpa_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+					simpa_repeat_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
+					simpa_repeat_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
 				}
 				for (k1 = 0; k1 < ARR_CNT(previous_solution_number); k1++) {
 					if (previous_solution_number[k1]) {
@@ -2601,10 +2763,11 @@ char	*cp;
 {
 	int	i, j;
 	int	i1;
+	int	found;
 	long	v;				/* Mathomatic variable */
 	double	d;
 	char	buf[MAX_CMD_LEN];
-	int	valid_range = false, repeat_flag = false, power_flag = false;
+	int	valid_range = false, power_flag = false;
 	char	*cp1;
 
 	if (strcmp_tospace(cp, "number") == 0) {
@@ -2681,17 +2844,31 @@ char	*cp;
 			}
 		}
 	} else {
-		v = 0;
 		do {
+			v = 0;
 			if (*cp) {
 				if ((cp = parse_var2(&v, cp)) == NULL) {
 					return false;
 				}
 			}
+			if (v) {
+				found = false;
+				for (i1 = i; i1 <= j; i1++) {
+					if (var_in_equation(i1, v)) {
+						found = true;
+						break;
+					}
+				}
+				if (!found) {
+					warning(_("Variable not found."));
+				}
+			}
 			for (i1 = i; i1 <= j; i1++) {
 				if (n_lhs[i1]) {
 					simpv_side(lhs[i1], &n_lhs[i1], v);
-					simpv_side(rhs[i1], &n_rhs[i1], v);
+					if (n_rhs[i1]) {
+						simpv_side(rhs[i1], &n_rhs[i1], v);
+					}
 				}
 			}
 		} while (*cp);
@@ -2713,18 +2890,18 @@ unfactor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	quick_flag = false, fully_flag = false, power_flag = false;
+	int	quick_flag = false, fraction_flag = false, power_flag = false;
 
 	for (;; cp = skip_param(cp)) {
-		if (strcmp_tospace(cp, "quick") == 0) {
+		if (strncasecmp(cp, "quick", 4) == 0) {
 			quick_flag = true;
 			continue;
 		}
-		if (strncasecmp(cp, "fully", 4) == 0) {
-			fully_flag = true;
+		if (strncasecmp(cp, "fraction", 4) == 0 || strncasecmp(cp, "fully", 4) == 0) {
+			fraction_flag = true;
 			continue;
 		}
-		if (strcmp_tospace(cp, "power") == 0) {
+		if (strncasecmp(cp, "power", 4) == 0) {
 			power_flag = true;
 			continue;
 		}
@@ -2733,30 +2910,31 @@ char	*cp;
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
+	partial_flag = !fraction_flag;
 	if (power_flag) {
 		for (; i <= j; i++) {
 			if (n_lhs[i]) {
-				if (fully_flag) {
-					uf_allpower(lhs[i], &n_lhs[i]);
-				} else {
+				if (quick_flag) {
 					uf_power(lhs[i], &n_lhs[i]);
+				} else {
+					uf_allpower(lhs[i], &n_lhs[i]);
 				}
 				elim_loop(lhs[i], &n_lhs[i]);
 				if (n_rhs[i]) {
-					if (fully_flag) {
-						uf_allpower(rhs[i], &n_rhs[i]);
-					} else {
+					if (quick_flag) {
 						uf_power(rhs[i], &n_rhs[i]);
+					} else {
+						uf_allpower(rhs[i], &n_rhs[i]);
 					}
 					elim_loop(rhs[i], &n_rhs[i]);
 				}
 				if (!return_result(i)) {
+					partial_flag = true;
 					return false;
 				}
 			}
 		}
 	} else {
-		partial_flag = !fully_flag;
 		for (; i <= j; i++) {
 			if (n_lhs[i]) {
 				if (quick_flag) {
@@ -2776,8 +2954,8 @@ char	*cp;
 				}
 			}
 		}
-		partial_flag = true;
 	}
+	partial_flag = true;
 	return true;
 }
 
@@ -2850,7 +3028,6 @@ char	*cp;
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
 			error(_("Can't open input file."));
-			printf(_("Read of file \"%s\" failed.\n"), cp);
 			return false;
 		}
 	} else {
@@ -2891,21 +3068,7 @@ FILE	*fp;
 		}
 	} else {
 		while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
-#if	LIBRARY
-			fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
-#else
-			default_color();
-			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
-			if (html_flag)
-				printf("<b>%s</b>", cp);	/* make input bold */
-			else
-				printf("%s", cp);
-			if (gfp != stdout) {
-				fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
-			}
-#endif
-			set_error_level(cp);
-			if (!process(cp)) {
+			if (!display_process(cp)) {
 				longjmp(jmp_save, 3);	/* jump to the above error trap */
 			}
 		}
@@ -2985,7 +3148,7 @@ char	*cp;
 		high_prec = true;
 		list_cmd("all");
 		high_prec = false;
-		gfp = stdout;
+		gfp = default_out;
 		fclose(fp);
 		rv = edit_sub(tmp_file);
 		unlink(tmp_file);
@@ -3042,7 +3205,7 @@ char	*cp;
 	high_prec = true;
 	rv = list_cmd("all");
 	high_prec = false;
-	gfp = stdout;
+	gfp = default_out;
 	if (fclose(fp))
 		rv = false;
 	if (rv) {
