@@ -1,7 +1,7 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -26,7 +26,7 @@ int
 roots_cmd(cp)
 char	*cp;
 {
-#define	MAX_ROOT	10000.0	/* root limit needed because more roots become more inaccurate and take longer to calculate */
+#define	MAX_ROOT	10000.0	/* Root limit needed because more roots become more inaccurate and take longer to check. */
 
 	complexs	c, c2;
 #if	!SILENT
@@ -36,6 +36,7 @@ char	*cp;
 	double		k, root, radius, theta, radius_root = 0.0;
 	char		buf[MAX_CMD_LEN];
 
+do_repeat:
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter root (positive integer): "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL)
@@ -69,7 +70,7 @@ char	*cp;
 		return false;
 	}
 	if (c.re == 0.0 && c.im == 0.0) {
-		return false;
+		return repeat_flag;
 	}
 /* convert to polar coordinates */
 	errno = 0;
@@ -106,22 +107,25 @@ char	*cp;
 			complex_fixup(&check);
 			printf(_("Inverse check:"));
 			if (check.re || check.im == 0.0) {
-				printf(" %.12g", check.re);
+				printf(" %.10g", check.re);
 			}
 			if (check.im) {
-				printf(" %+.12g*i", check.im);
+				printf(" %+.10g*i", check.im);
 			}
 			printf("\n\n");
 #endif
 		}
 	}
+	if (repeat_flag)
+		goto do_repeat;
 	return true;
 }
 
 /*
  * Approximate roots of complex numbers in an equation side:
- * (complex^real) and (real^complex) and (complex^complex).
+ * (complex^real) and (real^complex) and (complex^complex) all result in a complex number.
  * This only gives one root, even when there may be many roots.
+ * Works best when the equation side has been approximated before this.
  *
  * Return true and display a warning message if the equation side was modified.
  */
@@ -194,6 +198,19 @@ start_over:
 	return modified;
 }
 
+/*
+ * Approximate all roots of complex numbers in an equation side.
+ */
+void
+approximate_complex_roots(equation, np)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+{
+	do {
+		elim_loop(equation, np);
+	} while (complex_root_simp(equation, np));
+}
+
 /*
  * Get a constant, if the passed expression evaluates to a constant.
  * This should not be called from low level routines.
@@ -259,6 +276,8 @@ double		*dp;	/* pointer to returned double */
 
 /*
  * Get the value of a constant complex number expression.
+ * Doesn't always work unless expression is approximated first
+ * with something like the approximate command.
  *
  * If successful return true with complex number in *cp.
  */
