@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic differentiation routines and related commands.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -84,7 +84,7 @@ long		v;
 				break;
 			default:
 /* Oops.  More than one operator on the same level in this expression. */
-				error("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
+				error_bug("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
 				return false;
 			}
 			op = equation[endloc].token.operatr;
@@ -305,6 +305,9 @@ char	*cp;
 	}
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
+		if (!solved_equation(cur_equation)) {
+			warning(_("Not a solved equation.  Only the RHS will be differentiated."));
+		}
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
 		dest = rhs[i];
@@ -337,19 +340,21 @@ char	*cp;
 		}
 	}
 	if (no_vars(source, *nps, &v)) {
+#if	!SILENT
 		error(_("Current expression contains no variables; the derivative would be zero."));
 		return false;
+#endif
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
 	}
+#if	!SILENT
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
 		error(_("Variable not found; the derivative would be zero."));
 		return false;
 	}
-#if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
 		if (n_rhs[cur_equation]) {
@@ -357,8 +362,11 @@ char	*cp;
 		} else {
 			printf(_("Differentiating with respect to (%s)"), var_str);
 		}
+		if (order != 1) {
+			printf(_(" %ld times"), order);
+		}
 		if (simplify_flag) {
-			printf(" and simplifying...\n");
+			printf(_(" and simplifying...\n"));
 		} else {
 			printf("...\n");
 		}
@@ -491,11 +499,16 @@ char	*cp;
 	int		our_nlhs, our_nrhs;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
+	int		simplify_flag = true;
 
 	cp_start = cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	if (strcmp_tospace(cp, "nosimplify") == 0) {
+		simplify_flag = false;
+		cp = skip_param(cp);
+	}
 	i = next_espace();
 	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
@@ -531,18 +544,14 @@ char	*cp;
 		}
 		cp = cp1;
 	}
-	if (no_vars(source, *nps, &v)) {
-		error(_("Current expression contains no variables."));
-		return false;
-	}
+	no_vars(source, *nps, &v);
 	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
 	}
 	if (!found_var(source, *nps, v)) {
-		error(_("Variable not found."));
-		return false;
+		warning(_("Variable not found."));
 	}
 	blt(rhs[our], source, *nps * sizeof(token_type));
 	our_nrhs = *nps;
@@ -585,7 +594,14 @@ char	*cp;
 		}
 	}
 #if	!SILENT
-	printf(_("Computing...\n"));
+	printf(_("Computing the Taylor series"));
+	if (n_rhs[cur_equation]) {
+		printf(_(" of the RHS"));
+	}
+	if (simplify_flag) {
+		printf(_(" and simplifying"));
+	}
+	printf("...\n");
 #endif
 	n = 0;
 	i1 = 0;
@@ -650,7 +666,9 @@ loop_again:
 	n1 += 4;
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
-	uf_simp(dest, &n1);
+	if (simplify_flag) {
+		uf_simp(dest, &n1);
+	}
 	side_debug(1, dest, n1);
 	if (exp_contains_infinity(dest, n1)) {
 		error(_("Result invalid because it contains infinity or NaN."));
@@ -744,6 +762,9 @@ char	*cp;
 		error(_("Variable not found."));
 		return false;
 	}
+	if (*cp == '=') {
+		cp = skip_space(cp + 1);
+	}
 	if (*cp) {
 		input_column += (cp - cp_start);
 		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
@@ -758,15 +779,10 @@ char	*cp;
 	}
 /* copy the current equation to a new equation space, then simplify and work on the copy: */
 	copy_espace(cur_equation, i);
-#if	true
 	simpa_side(rhs[i], &n_rhs[i], false, false);
-#else
-	simple_frac_side(rhs[i], &n_rhs[i]);
-#endif
+
 /* see if the limit expression is positive infinity: */
 	simp_loop(tes, &n_tes);
-
-	debug_string(0, "Solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
@@ -782,7 +798,9 @@ char	*cp;
 		n_tlhs = 3;
 		subst_var_with_exp(rhs[i], &n_rhs[i], tlhs, n_tlhs, v);
 	}
+
 /* General limit taking, solve for the limit variable: */
+	debug_string(0, "Solving...");
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
