@@ -1,7 +1,7 @@
 /*
  * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -78,6 +78,7 @@ extern int		ucnt[];
 extern int		uno;
 
 extern sign_array_type	sign_array;
+extern FILE		*default_out;
 extern FILE		*gfp;
 extern jmp_buf		jmp_save;
 extern int		test_mode;
@@ -86,6 +87,7 @@ extern int		echo_input;
 extern volatile int	abort_flag;
 extern int		pull_number;
 extern int		security_level;
+extern int		repeat_flag;
 
 extern const char	*error_str;
 extern char		*result_str;
