@@ -1,7 +1,7 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -276,6 +276,8 @@ int		factor_flag;
  *
  * If "level_code" is 3, nothing is done.
  *
+ * Add 4 to "level_code" to always factor out the GCD of rational coefficients.
+ *
  * Return true if equation side was modified.
  */
 int
@@ -284,7 +286,7 @@ token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 int		level_code;	/* see above */
 {
-	if (level_code > 2)
+	if (level_code == 3)
 		return false;
 	return fc_recurse(equation, np, 0, 1, level_code);
 }
@@ -313,8 +315,8 @@ int		level_code;
 	}
 	if (modified)
 		return true;
-	improve_readability = (level_code > 1 || (level_code && (level == 1)));
-	gcd_flag = (improve_readability && integer_coefficients);
+	improve_readability = ((level_code & 3) > 1 || ((level_code & 3) && (level == 1)));
+	gcd_flag = ((improve_readability && integer_coefficients) || (level_code & 4));
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level == level) {
 			switch (equation[i].kind) {
@@ -348,7 +350,7 @@ int		level_code;
 			} else {
 				if (minimum > d)
 					minimum = d;
-				if (gcd_flag && cogcd)
+				if (gcd_flag && cogcd != 0.0)
 					cogcd = gcd_verified(d, cogcd);
 			}
 		} else {
@@ -376,7 +378,7 @@ int		level_code;
 						} else {
 							if (minimum > d)
 								minimum = d;
-							if (gcd_flag && cogcd)
+							if (gcd_flag && cogcd != 0.0)
 								cogcd = gcd_verified(d, cogcd);
 						}
 						i = j;
@@ -394,7 +396,7 @@ int		level_code;
 			} else {
 				if (minimum > 1.0)
 					minimum = 1.0;
-				if (gcd_flag && cogcd)
+				if (gcd_flag && cogcd != 0.0)
 					cogcd = gcd_verified(1.0, cogcd);
 			}
 			i = j;
@@ -402,7 +404,7 @@ int		level_code;
 		}
 		i++;
 	}
-	if (gcd_flag && cogcd && fmod(cogcd, 1.0) == 0.0) {
+	if (gcd_flag && cogcd != 0.0 /* && fmod(cogcd, 1.0) == 0.0 */) {
 		minimum = cogcd;
 	}
 	if (first || op_count == 0 || const_count > 1 || (!neg_flag && minimum == 1.0))
