@@ -3,7 +3,7 @@
  * C initializes global variables and arrays to zero by default.
  * This is required for proper operation.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  *
  * Most global variables for Mathomatic are defined here and duplicated in "externs.h".
  */
@@ -49,7 +49,7 @@ int		autosolve = true;			/* Allows solving by typing the variable name at the ma
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
 int		autoselect = true;			/* Allows selecting equation spaces by typing the number */
 char		special_variable_characters[256] = "\\"; /* user defined characters for variable names, 0 terminated */
-int		integer_coefficients = false;		/* if true, factor out the GCD of integer coefficients */
+int		integer_coefficients = false;		/* if true, factor out the GCD of rational coefficients */
 int		right_associative_power;		/* if true, evaluate power operators right to left */
 int		negate_highest_precedence = true;	/* if true, negation (-x) has the highest precedence */
 int		power_starstar;				/* if true, display power operator as "**", otherwise "^" */
@@ -58,7 +58,7 @@ int		debug_level;				/* current debug level */
 #endif
 
 /* variables having to do with color output mode */
-#if	LIBRARY
+#if	LIBRARY || NO_COLOR
 int		color_flag = false;		/* library shouldn't default to color mode */
 #else
 int		color_flag = true;		/* "set color" flag, true for color output mode */
@@ -96,13 +96,14 @@ int		uno;			/* number of unique factors stored in unique[] */
 
 /* misc. variables */
 sign_array_type	sign_array;		/* for keeping track of unique "sign" variables */
+FILE		*default_out;		/* file pointer where all gfp output goes by default */
 FILE		*gfp;			/* global output file pointer, for dynamically redirecting Mathomatic output */
 jmp_buf		jmp_save;		/* for setjmp(3) to longjmp(3) to when an error happens deep within this code */
 int		test_mode;		/* test mode flag (-t) */
 int		quiet_mode;		/* quiet mode (-q, don't display prompts) */
 int		echo_input;		/* if true, echo input and don't use readline */
 int		readline_enabled = true;	/* set to false (-r) to disable readline */
-int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fully" */
+int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fraction" */
 int		symb_flag;		/* true during "simplify symbolic", which is not 100% mathematically correct */
 int		symblify = true;	/* if true, set symb_flag when helpful during solving, etc. */
 int		high_prec;		/* flag to output constants in higher precision (used when saving equations) */
@@ -115,8 +116,9 @@ int		pull_number;		/* equation space number to pull when using the library */
 #if	!SECURE
 int		security_level;		/* current enforced security level for session  */
 #endif
+int		repeat_flag;		/* true if the command is to repeat its function */
 
 /* library variables go here */
-const char	*error_str;		/* last error string */
 char		*result_str;		/* returned result when using as library */
+const char	*error_str;		/* last error string */
 const char	*warning_str;		/* last warning string */
