@@ -3,7 +3,7 @@
  * Automatically includes all necessary C include files for
  * any Mathomatic C source code.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #define	true	1
@@ -19,10 +19,10 @@
 #endif
 
 #if 	CYGWIN
-#undef	UNIX
+#undef	UNIX		/* Unix desktop functionality is covered by defining CYGWIN */
 #elif	__CYGWIN__
 #warning Compiling under Cygwin without proper defines.
-#warning Please define "CYGWIN" on the compiler command line with "-DCYGWIN".
+#warning Please define CYGWIN on the compiler command line with -DCYGWIN
 #endif
 
 /* Include files from /usr/include: */
@@ -41,9 +41,12 @@
 #include <errno.h>
 #include <signal.h>
 #if	I18N
-#include <libintl.h>
+#include <libintl.h>	/* Mac OS X doesn't have libintl.h, so define "char *gettext();" then. */
+#include <locale.h>
 #endif
 #if	READLINE
+#include <curses.h>
+#include <term.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
