@@ -1,9 +1,7 @@
 /*
  * Mathomatic integration routines and commands.
  *
- * These are very low level, so they don't do much.
- *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -296,6 +294,9 @@ char	*cp;
 		} else {
 			printf(_("Integrating with respect to (%s)"), var_str);
 		}
+		if (integrate_order != 1.0) {
+			printf(_(" %.0f times"), integrate_order);
+		}
 		printf(_(" and simplifying...\n"));
 	}
 #endif
@@ -590,7 +591,11 @@ char	*cp;
 			return false;
 		}
 	}
+#if	true
 	simp_loop(dest, &n1);
+#else
+	simpa_side(dest, &n1, false, false);
+#endif
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
@@ -662,7 +667,7 @@ char	*cp;
 		}
 	}
 	if (singularity) {
-		warning(_("Singularity detected, result of numerical integration may be wrong."));
+		warning(_("Singularity detected, result of numerical integration might be wrong."));
 	}
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
