@@ -1,11 +1,7 @@
 /*
- * This file contains a test for the Mathomatic symbolic math library and API.
- * Refer to this, if you are going to use the Mathomatic code in other projects.
- *
- * To run this, compile and link with the Mathomatic symbolic math library
- * and create an executable.
- *
- * Written by George Gesslein II.
+ * This file contains the test/example program
+ * for the Mathomatic symbolic math library and API.
+ * Copy or refer to this, if you are going to use the Mathomatic code in other projects.
  */
 
 #include <stdio.h>
@@ -21,12 +17,13 @@ main(int argc, char **argv)
 	char		buf[1000];	/* buffer */
 
 	printf("Mathomatic library test program.\n");
-	if (!matho_init()) {
+	/* Initialize all global variables and arrays so that Mathomatic will work properly. */
+	if (!matho_init()) {	/* call this function exactly once in your program */
 		fprintf(stderr, "Not enough memory.\n");
 		exit(1);
 	}
 	/* Mathomatic is ready for use. */
-	/* This is an input/output loop for testing. */
+	/* This is a standard input/output loop for testing. */
 	for (;;) {
 		printf("%d-> ", cur_equation + 1);
 		if ((cp = fgets(buf, sizeof(buf), stdin)) == NULL)
