@@ -2,7 +2,7 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -10,8 +10,6 @@
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 #define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
-#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
-
 static int	flist();
 static int	flist_recurse();
 
@@ -58,19 +56,19 @@ static int	cur_pos;	/* current position in the current line on the screen */
 /*
  * Reset terminal attributes function.
  * Turn color off if color mode is on.
- * Subsequent output will have no color.
+ * Subsequent output will have no color until the next call to set_color().
  */
 void
 reset_attr()
 {
-	cur_color = -1;
-	if (color_flag) {
+	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			printf("</font>");
 		} else {
 			printf("\033[0m");
 		}
 	}
+	cur_color = -1;
 	if (fflush(NULL)) {	/* flush all output */
 		perror("fflush(3) call failed");
 	}
@@ -116,6 +114,27 @@ default_color()
 	set_color(0);
 }
 
+/*
+ * Trim the trailing zeros from a string, after the decimal point.
+ */
+static void
+trim_zeros(buf)
+char	*buf;
+{
+	int	j;
+
+	j = strlen(buf) - 1;
+	for (; j >= 0; j--) {
+		if (buf[j] == '0')
+			continue;
+		if (buf[j] == '.') {
+			buf[j+2] = '\0';
+		} else {
+			break;
+		}
+	}
+}
+
 /*
  * Display the expression or equation stored in equation space "n" in single-line format.
  * The equation space is not modified.
@@ -126,7 +145,7 @@ int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
 int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
-			/* 1 for Maxima, 2 for other */
+			/* 1 for Maxima, 2 for other, 3 for Gnuplot */
 {
 	int	len = 0;
 
@@ -201,13 +220,12 @@ int		n2;
  * Convert the passed Mathomatic variable to an ASCII variable name.
  * The ASCII variable name is stored in the global var_str[].
  *
- * Return length of variable name (number of ASCII characters).
+ * Return the length of the variable name string in var_str[].
+ * Nothing is displayed.
  *
  * If (lang_code == 0), use standard Mathomatic format.
- * If (lang_code == 1), make variable name C language compatible.
- * If (lang_code == 2), make variable name Java compatible.
- * If (lang_code == 3), make variable name Python compatible.
- * If lang_code is negative, create an exportable variable name: -1 for Maxima, -2 for other.
+ * If (lang_code > 0), make variable compatible with the language defined in the enumeration language_list defined in am.h
+ * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other.
  */
 int
 list_var(v, lang_code)
@@ -215,17 +233,20 @@ long	v;		/* variable to convert */
 int	lang_code;	/* language code */
 {
 	int		j;
-	char		*cp = NULL, buf[100];
+	char		*cp = NULL;
 
 	var_str[0] = '\0';
 	switch (v & VAR_MASK) {
 	case V_NULL:
-		return(0);
+		return(strlen(var_str));
 	case SIGN:
 		cp = "sign";
 		break;
 	case IMAGINARY:
 		switch (lang_code) {
+		case -3:
+			cp = "{0,1}";
+			break;
 		case -2:
 			cp = "i";
 			break;
@@ -235,29 +256,32 @@ int	lang_code;	/* language code */
 		case 0:
 			cp = "i#";
 			break;
-		case 3:
+		case PYTHON:
 			cp = "1j";
 			break;
 		default:
-			cp = "I";
+			cp = "1.0i";
 			break;
 		}
 		break;
 	case V_E:
 		switch (lang_code) {
+		case -3:
+			cp = "exp(1.0)";
+			break;
 		case -1:
 			cp = "%e";
 			break;
 		case 0:
 			cp = "e#";
 			break;
-		case 1:
+		case C:
 			cp ="M_E";
 			break;
-		case 2:
+		case JAVA:
 			cp = "Math.E";
 			break;
-		case 3:
+		case PYTHON:
 			cp = "math.e";
 			break;
 		default:
@@ -273,13 +297,13 @@ int	lang_code;	/* language code */
 		case 0:
 			cp = "pi#";
 			break;
-		case 1:
+		case C:
 			cp = "M_PI";
 			break;
-		case 2:
+		case JAVA:
 			cp = "Math.PI";
 			break;
-		case 3:
+		case PYTHON:
 			cp = "math.pi";
 			break;
 		default:
@@ -298,11 +322,11 @@ int	lang_code;	/* language code */
 		break;
 	}
 	if (cp) {
-		my_strlcpy(var_str, cp, sizeof(var_str));
 		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
 		if (j) {
-			snprintf(buf, sizeof(buf), "%d", j - 1);
-			strcat(var_str, buf);
+			snprintf(var_str, sizeof(var_str), "%s%d", cp, j - 1);
+		} else {
+			my_strlcpy(var_str, cp, sizeof(var_str));
 		}
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
@@ -322,101 +346,9 @@ list_proc(p1, n, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other */
+				/* 1 for Maxima, 2 for other, 3 for Gnuplot */
 {
-	int	i, j, k, i1;
-	int	min1;
-	int	cur_level;
-	char	*cp;
-	int	len = 0;
-
-	cur_level = min1 = min_level(p1, n);
-	for (i = 0; i < n; i++) {
-		j = cur_level - p1[i].level;
-		k = abs(j);
-		for (i1 = 1; i1 <= k; i1++) {
-			if (j > 0) {
-				cur_level--;
-				len += fprintf(gfp, ")");
-				set_color(cur_level-min1);
-			} else {
-				cur_level++;
-				set_color(cur_level-min1);
-				len += fprintf(gfp, "(");
-			}
-		}
-		switch (p1[i].kind) {
-		case CONSTANT:
-			if (p1[i].token.constant == 0.0) {
-				p1[i].token.constant = 0.0; /* fix -0 */
-			}
-			if (high_prec || export_flag) {
-				if (p1[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.20g)", p1[i].token.constant);
-				} else {
-					len += fprintf(gfp, "%.20g", p1[i].token.constant);
-				}
-			} else if (finance_option) {
-				if (p1[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.*f)", finance_option, p1[i].token.constant);
-				} else {
-					len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
-				}
-			} else {
-				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
-					len += fprintf(gfp, "(%.*g)", precision, p1[i].token.constant);
-				} else {
-					len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
-				}
-			}
-			break;
-		case VARIABLE:
-			list_var(p1[i].token.variable, 0 - export_flag);
-			len += fprintf(gfp, "%s", var_str);
-			break;
-		case OPERATOR:
-			cp = _("(unknown operator)");
-			switch (p1[i].token.operatr) {
-			case PLUS:
-				cp = " + ";
-				break;
-			case MINUS:
-				cp = " - ";
-				break;
-			case TIMES:
-				cp = "*";
-				break;
-			case DIVIDE:
-				cp = "/";
-				break;
-			case IDIVIDE:
-				cp = "//";
-				break;
-			case MODULUS:
-				cp = MODULUS_STRING;
-				break;
-			case POWER:
-				if (power_starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
-				break;
-			case FACTORIAL:
-				cp = "!";
-				i++;
-				break;
-			}
-			len += fprintf(gfp, "%s", cp);
-			break;
-		}
-	}
-	for (j = cur_level - min1; j > 0;) {
-		len += fprintf(gfp, ")");
-		j--;
-		set_color(j);
-	}
-	return len;
+	return list_string_sub(p1, n, true, NULL, export_flag);
 }
 
 /*
@@ -436,10 +368,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	if (n_lhs[n] <= 0) {
 		return NULL;
 	}
-	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
+	len = list_string(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
-		len += list_string_sub(rhs[n], n_rhs[n], NULL, export_flag);
+		len += list_string(rhs[n], n_rhs[n], NULL, export_flag);
 	}
 	len += 2;
 	cp = (char *) malloc(len);
@@ -447,10 +379,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
+	list_string(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
 		strcat(cp, EQUATE_STRING);
-		list_string_sub(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
+		list_string(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
 	}
 	if (export_flag == 1) {
 		strcat(cp, ";");
@@ -476,14 +408,14 @@ int		export_flag;
 	if (n <= 0) {
 		return NULL;
 	}
-	len = list_string_sub(p1, n, NULL, export_flag);
+	len = list_string(p1, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(p1, n, cp, export_flag);
+	list_string(p1, n, cp, export_flag);
 	return cp;
 }
 
@@ -491,25 +423,44 @@ int		export_flag;
  * Convert an expression to a text string and store in "string" if
  * "string" is not NULL.  "string" need not be initialized,
  * but must be long enough to contain the expression.
+ * To find the storage size needed, call with "string" set to NULL first.
  *
  * Return length (number of characters).
  */
 int
-list_string_sub(p1, n, string, export_flag)
+list_string(p1, n, string, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
-char		*string;	/* buffer or NULL pointer */
+char		*string;	/* buffer to save output to or NULL pointer */
 int		export_flag;
 {
+	return list_string_sub(p1, n, false, string, export_flag);
+}
+
+int
+list_string_sub(p1, n, outflag, string, export_flag)
+token_type	*p1;		/* expression pointer */
+int		n;		/* length of expression */
+int		outflag;	/* if true, output to gfp */
+char		*string;	/* buffer to save output to or NULL pointer */
+int		export_flag;
+{
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
+
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
 	char	*cp;
 	int	len = 0;
-	char	buf[500];
+	char	buf[500], buf2[500];
+	int	export_precision;
 
 	if (string)
 		string[0] = '\0';
+	if (high_prec)
+		export_precision = 20;
+	else
+		export_precision = DBL_DIG;
 	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
 		j = cur_level - p1[i].level;
@@ -518,8 +469,12 @@ int		export_flag;
 			if (j > 0) {
 				cur_level--;
 				APPEND(")");
+				if (outflag && !export_flag)
+					set_color(cur_level-min1);
 			} else {
 				cur_level++;
+				if (outflag && !export_flag)
+					set_color(cur_level-min1);
 				APPEND("(");
 			}
 		}
@@ -528,26 +483,28 @@ int		export_flag;
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (high_prec || export_flag) {
-				if (p1[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.20g)", p1[i].token.constant);
-				} else {
-					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
-				}
+			if (export_flag == 3) {
+				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, p1[i].token.constant);
+				trim_zeros(buf);
+			} else if (export_flag || high_prec) {
+				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
 			} else if (finance_option) {
-				if (p1[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
-				} else {
-					snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
-				}
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
 				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
 					snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
 				} else {
 					snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 				}
+				APPEND(buf);
+				break;
+			}
+			if (p1[i].token.constant < 0.0) {
+				snprintf(buf2, sizeof(buf2), "(%s)", buf);
+				APPEND(buf2);
+			} else {
+				APPEND(buf);
 			}
-			APPEND(buf);
 			break;
 		case VARIABLE:
 			list_var(p1[i].token.variable, 0 - export_flag);
@@ -575,7 +532,7 @@ int		export_flag;
 				cp = MODULUS_STRING;
 				break;
 			case POWER:
-				if (power_starstar) {
+				if (power_starstar || export_flag == 3) {
 					cp = "**";
 				} else {
 					cp = "^";
@@ -590,8 +547,11 @@ int		export_flag;
 			break;
 		}
 	}
-	for (j = cur_level - min1; j > 0; j--) {
+	for (j = cur_level - min1; j > 0;) {
 		APPEND(")");
+		j--;
+		if (outflag && !export_flag)
+			set_color(j);
 	}
 	return len;
 }
@@ -608,9 +568,19 @@ int		n;		/* length of expression */
 	int	i;
 
 	for (i = 0; i < n; i++) {
-		if ((p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
-		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
-			return false;
+		switch (p1[i].kind) {
+		case CONSTANT:
+			if (fmod(p1[i].token.constant, 1.0)) {
+				return false;
+			}
+			break;
+		case VARIABLE:
+			if (p1[i].token.variable < IMAGINARY) {
+				return false;
+			}
+			break;
+		case OPERATOR:
+			break;
 		}
 	}
 	return true;
@@ -623,9 +593,9 @@ int		n;		/* length of expression */
  */
 int
 list_c_equation(en, language, int_flag)
-int	en;		/* equation space number */
-int	language;
-int	int_flag;	/* integer arithmetic flag */
+int			en;		/* equation space number */
+enum language_list	language;
+int			int_flag;	/* integer arithmetic flag */
 {
 	int	len = 0;
 
@@ -636,8 +606,13 @@ int	int_flag;	/* integer arithmetic flag */
 		len += fprintf(gfp, " = ");
 		len += list_code(rhs[en], &n_rhs[en], language, int_flag);
 	}
-	if (language != 2) {
+	switch (language) {
+	case C:
+	case JAVA:
 		len += fprintf(gfp, ";");
+		break;
+	default:
+		break;
 	}
 	fprintf(gfp, "\n");
 	return len;
@@ -651,10 +626,10 @@ int	int_flag;	/* integer arithmetic flag */
  */
 int
 list_code(equation, np, language, int_flag)
-token_type	*equation;	/* equation side pointer */
-int		*np;		/* pointer to length of equation side */
-int		language;	/* if 0, generate C code; if 1, generate Java code; if 2, generate Python code */
-int		int_flag;	/* integer arithmetic flag, should work with any language */
+token_type		*equation;	/* equation side pointer */
+int			*np;		/* pointer to length of equation side */
+enum language_list	language;	/* see enumeration language_list in am.h */
+int			int_flag;	/* integer arithmetic flag, should work with any language */
 {
 	int	i, j, k, i1, i2;
 	int	min1;
@@ -687,12 +662,17 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 								equation[i2].token.operatr = TIMES;
 								equation[i2+1] = equation[i2-1];
 							} else {
-								if (int_flag || language > 1)
-									break;
-								if (language) {
-									len += fprintf(gfp, "Math.pow");
-								} else {
-									len += fprintf(gfp, "pow");
+								if (!int_flag) {
+									switch (language) {
+									case C:
+										len += fprintf(gfp, "pow");
+							 			break;
+									case JAVA:
+										len += fprintf(gfp, "Math.pow");
+										break;
+									default:
+										break;
+									}
 								}
 							}
 							break;
@@ -716,16 +696,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
 				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, equation[i].token.constant);
-				j = strlen(buf) - 1;
-				for (; j >= 0; j--) {
-					if (buf[j] == '0')
-						continue;
-					if (buf[j] == '.') {
-						buf[j+2] = '\0';
-					} else {
-						break;
-					}
-				}
+				trim_zeros(buf);
 			}
 			if (equation[i].token.constant < 0) {
 				len += fprintf(gfp, "(%s)", buf);
@@ -734,8 +705,12 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			}
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, language + 1);
-			fprintf(gfp, "%s", var_str);
+		  	if (int_flag && (language == C || language == JAVA) && equation[i].token.variable == IMAGINARY) {
+				len += fprintf(gfp, "1i");
+			} else {
+				list_var(equation[i].token.variable, language);
+				len += fprintf(gfp, "%s", var_str);
+			}
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
@@ -757,7 +732,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = MODULUS_STRING;
 				break;
 			case POWER:
-				if (int_flag || language > 1) {
+				if (int_flag || language == PYTHON) {
 					cp = "**";
 				} else {
 					cp = ", ";
@@ -1010,7 +985,7 @@ check_again:
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
 					if (html_out) {
-						fprintf(gfp, "&mdash;");
+						fprintf(gfp, "&minus;");
 					} else {
 						fprintf(gfp, "-");
 					}
