@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -470,6 +470,8 @@ parse_equation(n, cp)
 int	n;	/* equation space number */
 char	*cp;	/* pointer to the beginning of the equation character string */
 {
+	if (cp == NULL)
+		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
@@ -498,6 +500,8 @@ char		*cp;		/* string to parse */
 	int	i;
 	char	*cp1, *cp2;
 
+	if (cp == NULL)
+		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
@@ -587,6 +591,9 @@ char	*cp;
 	if (strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
+	} else if (strcasecmp(buf, "nan") == 0) {
+		error(_("NaN (Not a Number) cannot be directly entered."));
+		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 	} else {
@@ -701,11 +708,9 @@ char	*cp;	/* input string */
 
 	point_flag = true;
 /* handle comments, newlines, and the DOS EOF character (control-Z) by truncating the string where found */
-	for (cp1 = cp; *cp1; cp1++) {
-		if (*cp1 == ';' || *cp1 == '\n' || *cp1 == '\r' || *cp1 == '\032') {
-			*cp1 = '\0';
-			break;
-		}
+	cp1 = strpbrk(cp, ";\n\r\032");
+	if (cp1) {
+		*cp1 = '\0';
 	}
 	remove_trailing_spaces(cp);
 /* set point_flag to false if non-printable characters encountered */
