@@ -6,7 +6,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgen.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
@@ -41,8 +40,6 @@ main(int argc, char *argv[])
 {
 	struct winsize	ws;
 
-        prog_name = strdup(basename(argv[0]));
-
 	ws.ws_col = 0;
 	ws.ws_row = 0;
 	ioctl(1, TIOCGWINSZ, &ws);
@@ -77,7 +74,7 @@ main(int argc, char *argv[])
 }
 
 void
-allocate_triangle()
+allocate_triangle(void)
 {
 	int	i;
 
@@ -91,7 +88,7 @@ allocate_triangle()
 }
 
 void
-calculate_triangle()
+calculate_triangle(void)
 {
 	int	i, j;
 
@@ -107,7 +104,7 @@ calculate_triangle()
 }
 
 void
-display_triangle()
+display_triangle(void)
 {
 	int		i, j;
 	int		len;
@@ -172,7 +169,7 @@ center_buf(int line_number, int cell_size)
 }
 
 void
-usage()
+usage(void)
 {
 	printf("Usage: %s [number-of-lines]\n\n", prog_name);
 	printf("Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
