@@ -145,7 +145,11 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			errno = 0;
 			d1 = strtol(argv[i], &cp, 10);
-			if (errno || argv[i][0] == '\0' || *cp || d1 <= 0) {
+			if (errno) {
+				perror(NULL);
+				exit(1);
+			}
+			if (argv[i][0] == '\0' || *cp || d1 <= 0) {
 				fprintf(stderr, "Invalid number: \"%s\".\n", argv[i]);
 				exit(1);
 			}
