@@ -1,7 +1,7 @@
 /*
  * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -32,6 +32,7 @@ token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
+		printf("Bad expression size = %d.\n", *np);
 		error_bug("Internal error: organize() called with bad expression size.");
 	}
 	if (*np > n_tokens) {
@@ -154,7 +155,7 @@ token_type	*equation;	/* pointer to the beginning of equation side to simplify *
 int		*np;		/* pointer to length of equation side */
 {
 	if (abort_flag) {
-		/* Control-C pressed, return to main prompt and leave unsimplified */
+		/* Control-C pressed, gracefully return to main prompt and leave unsimplified */
 		abort_flag = false;
 		longjmp(jmp_save, 13);
 	}
@@ -223,14 +224,15 @@ int	n;	/* equation space number to simplify */
 }
 
 /*
- * For quick, mid-range simplification of an expression.
+ * For quick, mid-range simplification of an equation side.
+ * Trivial factoring is done.
  */
 void
 simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, 1.0, true, true, 2 /* was 1 */);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 6 /* was 2 */);
 }
 
 /*
@@ -245,7 +247,7 @@ int		zsolve;		/* true for solving for zero */
 	elim_loop(equation, np);
 	simp_constant_power(equation, np);
 	do {
-		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
+		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6 /* was 2 */);
 	} while (super_factor(equation, np, 0));
 }
 
@@ -260,7 +262,7 @@ long		v;		/* variable to factor, 0 for all variables */
 {
 	if (*np == 0)
 		return;
-	simp_ssub(equation, np, v, 0.0, v == 0, true, 2);
+	simp_ssub(equation, np, v, 0.0, v == 0, true, 6 /* was 2 */);
 }
 
 /*
@@ -274,9 +276,7 @@ int		*np;		/* pointer to equation side length */
 long		v;		/* variable to factor */
 {
 	if (v == IMAGINARY) {
-		do {
-			elim_loop(equation, np);
-		} while (complex_root_simp(equation, np));
+		approximate_complex_roots(equation, np);
 	}
 	do {
 		do {
@@ -500,24 +500,27 @@ simple_frac_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 {
-	if (*np == 0)
+	if (*np <= 0)
 		return;
+
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+				simp_ssub(equation, np, 0L, 1.0, false, true, 5);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
+	side_debug(2, equation, *np);
+
 	make_fractions(equation, np);
 	uf_tsimp(equation, np);
-	poly_factor(equation, np);
+	poly_factor(equation, np, true);
 	simpb_side(equation, np, true, false, 2);
+	fractions_and_group(equation, np);
 }
 
 /*
- * Main simplify code used by the simplify command.
- * This is the slowest and most thorough simplify of all.
+ * This is the slow and thorough simplify.
  * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
  * then does generalized polynomial simplifications.
  *
@@ -579,7 +582,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 2);
+				simp_ssub(equation, np, 0L, 1.0, false, true, 6 /* was 2 */);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
@@ -624,7 +627,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 			simpb_side(equation, np, false, true, 3);
 		}
 		/* factor polynomials */
-		if (!flag && poly_factor(equation, np)) {
+		if (!flag && poly_factor(equation, np, true)) {
 			flag = true;
 			simpb_side(equation, np, false, true, 3);
 			continue;
@@ -637,9 +640,9 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 		}
 		break;
 	}
-	simpb_side(equation, np, true, true, 1);
+/*	simpb_side(equation, np, true, true, 1); */
 	simp_constant_power(equation, np);
-	simp_loop(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 5 /* was 1 */);
 	unsimp_power(equation, np);
 	make_fractions(equation, np);
 	factor_power(equation, np);
@@ -648,10 +651,39 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
 	simpb_side(equation, np, true, true, 3);
-	poly_factor(equation, np);
+	poly_factor(equation, np, false);
 	simpb_side(equation, np, true, !frac_flag, 2);
 }
 
+/*
+ * This routine is used by the simplify command,
+ * and is the slowest and most thorough simplify of all.
+ * It repeats the simplification until the smallest expression is achieved,
+ * if repeat_flag is true.
+ *
+ * Globals tes[], tlhs[], and trhs[] are wiped out.
+ */
+void
+simpa_repeat_side(equation, np, quick_flag, frac_flag)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
+int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
+{
+	simpa_side(equation, np, quick_flag, frac_flag);
+	if (repeat_flag) {
+		do {
+			n_tes = *np;
+			blt(tes, equation, n_tes * sizeof(token_type));
+			simpa_side(equation, np, quick_flag, frac_flag);
+		} while (*np < n_tes);
+		if (*np != n_tes) {
+			*np = n_tes;
+			blt(equation, tes, n_tes * sizeof(token_type));
+		}
+	}
+}
+
 /*
  * Commonly used quick simplify routine that doesn't factor.
  *
@@ -1668,8 +1700,8 @@ no_op2:
 			opa2[oc2] = pv1 + 1;
 			used[oc2] = false;
 			if (++oc2 >= ARR_CNT(opa2)) {
-				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS may be too small.");
-				return false;	/* expression too big to compare */
+				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS exceeded.");
+				return false;
 			}
 		}
 	}
@@ -2047,7 +2079,8 @@ int		i1, i2;
 }
 
 /*
- * Recursively check and possibly reorder a "level" subexpression which starts at "loc" in an equation side.
+ * Recursively check and possibly reorder a "level" subexpression
+ * which starts at "loc" in an equation side.
  */
 static int
 order_recurse(equation, np, loc, level)
@@ -2151,7 +2184,7 @@ int		*np, loc, level;
 	}
 	return modified;
 corrupt:
-	error_bug("Expression is corrupt!  Please send a bug report.");
+	error_bug("Internal representation of expression is corrupt!");
 	return modified;	/* not reached */
 }
 
