@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic solve routines.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -14,6 +14,7 @@ static int	quad_solve();
 static int	increase();
 
 static int	repeat_count;
+static int	last_int_var = 0;
 
 /*
  * Solve using equation spaces.
@@ -984,7 +985,6 @@ big_bbreak:
 	len += alen;
 	if (found_var(scratch, len, v))
 		return false;
-	debug_string(1, _("Plugging equation into the quadratic formula:"));
 	blt(tlhs, x1_storage, nx1 * sizeof(token_type));
 	n_tlhs = nx1;
 	simp_loop(tlhs, &n_tlhs);
@@ -995,15 +995,7 @@ big_bbreak:
 	uf_tsimp(trhs, &n_trhs);	/* don't unfactor result so much, just unfactor what will be unfactored anyway */
 	simps_side(trhs, &n_trhs, false);
 	list_tdebug(1);
-#if	!SILENT
-	if (high_power == 2.0) {
-		fprintf(gfp, _("Equation was quadratic.\n"));
-	} else if (high_power == 4.0) {
-		fprintf(gfp, _("Equation was biquadratic.\n"));
-	} else {
-		fprintf(gfp, _("Equation was solved with the quadratic formula.\n"));
-	}
-#endif
+	debug_string(0, _("Equation was solved with the quadratic formula."));
 	return true;
 }
 
@@ -1031,6 +1023,8 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	double		numerator, denominator;
 	static int	prev_n1, prev_n2;
 	double		d1, d2;
+	complexs	c1, c2;
+	char		var_name_buf[MAX_VAR_LEN];
 
 	oldn = *side1np;
 	ep = &side1p[oldn];
@@ -1070,16 +1064,35 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	}
 	operandn = p1 - operandp;
 	if (op == POWER && operandp == side1p) {
-		if (!get_constant(side2p, *side2np, &d1))
+		if (!parse_complex(side2p, *side2np, &c1))
 			return false;
-		if (!get_constant(operandp, operandn, &d2))
+		if (!parse_complex(operandp, operandn, &c2))
 			return false;
 		debug_string(1, _("Taking logarithm of both equation sides:"));
-		*side2np = 1;
-		side2p->level = 1;
-		side2p->kind = CONSTANT;
 		errno = 0;
-		side2p->token.constant = log(d1) / log(d2);
+		c1 = complex_div(complex_log(c1), complex_log(c2));
+		*side2np = 0;
+		side2p[*side2np].level = 1;
+		side2p[*side2np].kind = CONSTANT;
+		side2p[*side2np].token.constant = c1.re;
+		(*side2np)++;
+		side2p[*side2np].level = 1;
+		side2p[*side2np].kind = OPERATOR;
+		side2p[*side2np].token.operatr = PLUS;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = CONSTANT;
+		side2p[*side2np].token.constant = c1.im;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = OPERATOR;
+		side2p[*side2np].token.operatr = TIMES;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = VARIABLE;
+		side2p[*side2np].token.variable = IMAGINARY;
+		(*side2np)++;
+
 		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
 		*side1np -= operandn + 1;
 		check_err();
@@ -1160,7 +1173,12 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2++;
 		p2->level = 2;
 		p2->kind = VARIABLE;
-		parse_var(&p2->token.variable, V_INTEGER_NAME);
+		snprintf(var_name_buf, sizeof(var_name_buf), "%s%.0d", V_INTEGER_NAME, last_int_var);
+		parse_var(&p2->token.variable, var_name_buf);
+		last_int_var++;
+		if (last_int_var < 0) {
+			last_int_var = 0;
+		}
 		p2++;
 		p2->level = 2;
 		p2->kind = OPERATOR;
