@@ -1,7 +1,7 @@
 /*
  * Group and combine denominators symbolically for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 #include "includes.h"
@@ -94,11 +94,13 @@ int		*np;		/* pointer to length of equation side */
 
 /*
  * Make equation side ready for display.
+ * Converts non-integer constants in an equation side to fractions,
+ * when exactly equal to simple fractions.
  *
  * Return true if any fractions were created.
  */
 int
-display_fractions_and_group(equation, np)
+fractions_and_group(equation, np)
 token_type	*equation;
 int		*np;
 {
@@ -112,17 +114,23 @@ int		*np;
 
 /*
  * This function is the guts of the display command.
+ *
+ * Return true if any fractions were created.
  */
-void
+int
 make_fractions_and_group(n)
 int	n;	/* equation space number */
 {
+	int	rv;
+
 	if (n_lhs[n] <= 0)
-		return;
-	display_fractions_and_group(lhs[n], &n_lhs[n]);
+		return false;
+	rv = fractions_and_group(lhs[n], &n_lhs[n]);
 	if (n_rhs[n]) {
-		display_fractions_and_group(rhs[n], &n_rhs[n]);
+		if (fractions_and_group(rhs[n], &n_rhs[n]))
+			rv = true;
 	}
+	return rv;
 }
 
 /*
@@ -137,14 +145,15 @@ int	n;	/* equation space number */
  *
  * If start_flag is 1, always combine denominators.
  * This can easily make an expression large and complicated.
+ * Used when solving for zero.
  *
  * If start_flag is 2, always combine denominators
  * and reduce the result by removing any polynomial GCD between them.
  * Note that this wipes out globals tlhs[] and trhs[].
  * This simplifies all algebraic fractions into a single simple fraction
- * and prevents making a more complicated (or larger) algebraic fraction.
+ * and usually prevents making a more complicated (or larger) algebraic fraction.
  *
- * Return true if equation side was modified.
+ * Return true if the equation side was modified.
  */
 int
 super_factor(equation, np, start_flag)
@@ -172,11 +181,7 @@ int		*np, loc, level, start_flag;
 
 	if (!start_flag) {
 		for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-#if	true
 			if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
-#else
-			if (equation[i].level == level && (equation[i].token.operatr == DIVIDE || equation[i].token.operatr == POWER)) {
-#endif
 				start_flag = true;
 				break;
 			}
